/*
 * Copyright (C) 2007 Red Hat, Inc.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 *
 *   Free Software Foundation
 *   51 Franklin Street, Fifth Floor
 *   Boston, MA  02111-1301
 *   USA.
 */

#ifndef aabag_h
#define aabag_h
#ident "$Id: aabag.h,v 1.3 2007/05/30 12:28:57 nalin Exp $"

#include <nspr.h>
#include <nss.h>
#include <certt.h>
#include "pkinit.h"

struct aa_bag;
struct module_context;

struct aa_bag *aa_bag_new(struct module_context *mcontext,
			  CERTCertDBHandle *certdb,
			  PK11PasswordFunc pwcb);
void aa_bag_free(struct aa_bag *bag);

SECStatus aa_bag_load_key_file(struct aa_bag *bag, const char *path,
			       struct pkinit_pwcb_args *pwcb_args);
SECStatus aa_bag_load_key_dir(struct aa_bag *bag, const char *path,
			      const char *pattern,
			      struct pkinit_pwcb_args *pwcb_args);
SECStatus aa_bag_load_cert_file(struct aa_bag *bag, const char *path,
			        struct pkinit_pwcb_args *pwcb_args);
SECStatus aa_bag_load_cert_dir(struct aa_bag *bag, const char *path,
			       const char *pattern,
			       struct pkinit_pwcb_args *pwcb_args);
SECStatus aa_bag_load_cacert_file(struct aa_bag *bag, const char *path,
				  struct pkinit_pwcb_args *pwcb_args);
SECStatus aa_bag_load_cacert_dir(struct aa_bag *bag, const char *path,
			         const char *pattern,
				 struct pkinit_pwcb_args *pwcb_args);
SECStatus aa_bag_add_cert_search_dir(struct aa_bag *bag, const char *path);
SECStatus aa_bag_add_cacert_search_dir(struct aa_bag *bag, const char *path);

SECKEYPrivateKeyList *aa_bag_list_loaded_keys(struct aa_bag *bag);
CERTCertList *aa_bag_list_loaded_certs(struct aa_bag *bag);
CERTCertList *aa_bag_find_cert_by_subject(struct aa_bag *bag,
					  SECItem *der_subject,
					  struct pkinit_pwcb_args *pwcb_args);
#endif
