/*
 * Copyright (C) 2006,2007 Red Hat, Inc.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the 
 *
 *   Free Software Foundation
 *   51 Franklin Street, Fifth Floor
 *   Boston, MA  02111-1301
 *   USA.
 */

#ifndef certs_h
#define certs_h

#ident "$Id: certs.h,v 1.19 2007/06/08 21:44:27 nalin Exp $"
#include "aabag.h"
#include "bcmst.h"
#include "pkinit.h"

struct pkinit_principal_name {
	SECItem name_type;
	SECItem **name_string;
};

struct pkinit_krb5_principal_name {
	SECItem realm;
	struct pkinit_principal_name principal_name;
};

struct other_name {
	SECItem oid;
	SECItem data;
};

enum subject_alt_name_type {
	subject_alt_name_other_name = 1,
	subject_alt_name_rfc822_name = 2,
	subject_alt_name_dnsname = 3,
};

struct subject_alt_name {
	int subject_alt_name_type;
	struct {
		struct other_name other_name;
		SECItem rfc822_name;
		SECItem dnsname;
	} subject_alt_name_value;
};

#define	CERT_MATCHES_UPN_SAN		(1 << 0)
#define	CERT_MATCHES_KRB_SAN		(1 << 1)
#define	CERT_MATCHES_DN_LIST		(1 << 2)
#define	CERT_MATCHES_TRUSTED_SERVERS	(1 << 3)
#define CERT_NAME_MASK			(CERT_MATCHES_UPN_SAN |\
					 CERT_MATCHES_KRB_SAN |\
					 CERT_MATCHES_DN_LIST |\
					 CERT_MATCHES_TRUSTED_SERVERS)
#define	CERT_MATCHES_CLIENT_AUTH_EKU	(1 << 4)
#define	CERT_MATCHES_SERVER_AUTH_EKU	(1 << 5)
#define	CERT_MATCHES_PKINIT_CLIENT_EKU	(1 << 6)
#define	CERT_MATCHES_PKINIT_KDC_EKU	(1 << 7)
#define	CERT_MATCHES_MS_SMARTCARD_EKU	(1 << 8)
#define CERT_EKU_MASK			(CERT_MATCHES_CLIENT_AUTH_EKU |\
					 CERT_MATCHES_PKINIT_CLIENT_EKU |\
					 CERT_MATCHES_PKINIT_KDC_EKU |\
					 CERT_MATCHES_MS_SMARTCARD_EKU)

#define CERT_MATCHES_DC_CERT_TEMPLATE	(1 << 9)
#define CERT_MATCHES_TRUSTED_GUID	(1 << 10)

CERTCertificate *cert_find_preferred_cert(struct module_context *mcontext,
					  PLArenaPool *pool,
					  PK11SlotInfo *preferred_slot,
					  struct aa_bag *bag,
					  struct pkinit_pwcb_args *pwcb_args,
					  struct external_principal_identifier **restricted_ca_list,
					  struct external_principal_identifier **banned_cert_list,
					  const char **additional_dn_list,
					  krb5_context kcontext,
					  krb5_principal principal,
					  PRInt32 cert_flags);
krb5_error_code cert_validate_client_certificate(struct module_context *mcontext,
						 PLArenaPool *pool,
						 CERTCertDBHandle *certdb,
						 struct aa_bag *bag,
						 CERTCertificate *cert,
						 SECItem **additional_certs,
						 struct pkinit_pwcb_args *,
						 const char **additional_dn_list,
						 PRInt32 cert_flags,
						 krb5_context kcontext,
						 krb5_principal client,
						 struct external_principal_identifier ***error_epis);
krb5_error_code cert_validate_kdc_certificate(struct module_context *mcontext,
					      PLArenaPool *pool,
					      CERTCertDBHandle *certdb,
					      struct aa_bag *bag,
					      CERTCertificate *kdc_cert,
					      SECItem **additional_certs,
					      struct pkinit_pwcb_args *,
					      const char **additional_dn_list,
					      PRInt32 cert_flags,
					      krb5_context kcontext,
					      krb5_principal server);

const SECItem *cert_get_oid_dh_public_number(void);
const SECItem *cert_get_oid_dh_key_agreement(void);
const SECItem *cert_get_oid_pkinit_auth_data(void);
const SECOidData *cert_get_oid_pkinit_auth_data_oid(void);
const SECItem *cert_get_oid_pkinit_rkey_data(void);
const SECOidData *cert_get_oid_pkinit_rkey_data_oid(void);
const SECItem *cert_get_oid_pkinit_dhkey_data(void);

SECItem **cert_get_guids(struct module_context *mcontext,
			 CERTCertificate *cert);

PRBool cert_is_domain_controller(struct module_context *mcontext,
				 CERTCertificate *signer,
				 krb5_context kcontext,
				 krb5_principal tgt_principal);
krb5_error_code cert_verify_cert_for_encryption(struct module_context *mcontext,
						CERTCertDBHandle *certdb,
						struct aa_bag *bag,
						SECItem *client);
CERTCertificate *cert_find_cert_issuer(CERTCertificate *cert, int64 valid_time,
				       SECCertUsage usage, struct aa_bag *bag);

SECKEYPrivateKey *cert_find_key_for_cert(struct module_context *mcontext,
					 PK11SlotInfo *preferred_slot,
					 CERTCertificate *cert,
					 struct aa_bag *bag,
					 struct pkinit_pwcb_args *pwcb_args);

#endif
